#
# ZETA.pm - interface description file for perl writers
#
# ZETA Perl - Perl binding for the ANSI/NISO Z39.50-1995 Protocol
# Copyright (c) 1996, 1999 Finsiel S.p.A.
#
# Rocco Carbone <rocco@tecsiel.it> 4Q 1996
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#


#
# The name of the game!
#
package ZETA;

$PKGNAME        = 'ZETA Perl';
$MODULE         = 'Extension Module for Perl 5';
$VERSION        = '0.059';
$VERBOSE        = 0;
$TIMEOUT        = 360;

my $RELEASED    = '19 Feb 1999';
my $DESCRIPTION = 'Simple access to Z39.50 V3 services via Perl 5 scripts';
my $COPYRIGHT   = 'Copyright (c) Finsiel S.p.A. 1996, 1999. Released under GNU/GPL.';

#
# What we need is...
#
require 5.003;

require Exporter;
require DynaLoader;
require AutoLoader;

#
# Perl pragmas to restrict unsafe constructs
#
use strict;

#
# Handy routines to throw exceptions outside of the current package
use Carp;

#
# Perl pragmas to predeclare global variable names
#
use vars qw ($z_errno $PKGNAME $MODULE $VERSION @ISA @EXPORT $AUTOLOAD);

@ZETA::ISA = qw (Exporter DynaLoader AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

@ZETA::EXPORT = qw (zetaversion $PKGNAME $MODULE $VERSION
                    z_trace_off z_trace_on z_version);

@ZETA::EXPORT_OK = qw ();

sub AUTOLOAD
  {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0)
      {
	if ($! =~ /Invalid/)
	  {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	  }
	else
	  {
	    croak "Your vendor has not defined ZETA macro $constname";
	  }
      }
    eval "sub $AUTOLOAD { $val }";
    goto &$AUTOLOAD;
  }

bootstrap ZETA $VERSION;

# Preloaded methods go here.

$z_errno = 0;

sub zetaversion ()
  {
    my $hardware = `uname -m`;
    my $software = `uname -s`;
    my $version  = `uname -r`;
    my $node     = `uname -n`;

    chop ($hardware);
    chop ($software);
    chop ($version);
    chop ($node);

    my $you   = getlogin ();
    my $where = $hardware."-".$software."-".$version." "."(".$node.")";

    print "\nThis is:\n";

    print "\t$PKGNAME v. $VERSION ($MODULE) of $RELEASED\n";
    print "\n\t$COPYRIGHT\n";

    print "\nIt provides:\n\t$DESCRIPTION\n";

    print "\tStarted by $you under $where\n";
  }


#
# The basic operations offered by Z39.50 are provided through an object
# oriented interface for modularity and ease of use.
# The primary object is ZETA::Connection which is implemented as
# a blessed hash reference.
#
# The object supports the following method calls:
# 'Init', 'Search', 'Present', 'Close', 'Delete', 'Scan', and 'Sort'
#

package ZETA::Connection;

#
# The costructor of the object
#
sub New
  {
    my $host = shift;
    my $port = shift;

    my $this  = {};
    my ($name, $aliases, $type, $len, $thisaddr);

    %$this = @_;

    #
    # destination host and port default to localhost and 2100
    #
    $this->{host} ||= $host ? $host : 'localhost';
    $this->{port} ||= $port ? $port : 2100;

    #
    # convert to dotted ip address if needed
    #
    if ($this->{host} =~ /\d+\.\d+\.\d+\.\d+/)
      {
	$this->{address} = $this->{host};
      }
    else
      {
	($name, $aliases, $type, $len, $thisaddr) =
	  gethostbyname ($this->{host});
	if ($name)
	  {
	    $this->{address} = join ('.', unpack ("C4", $thisaddr));
	  }
      }

    warn (" Warning: Unresolved destination address $this->{host}\n")
      unless $this->{address};

    $this->{strerror} = ''; # if methods return undef check for expln.
    $this->{errno}    = 0;  # contains ZETA error return

    #
    # Establish a network connection with server
    #
    $this->{sessionid} = ZETA::_connect ($this->{host}, $this->{port});

    #
    # Protocol version v2 or v3, defaults to v3
    #
    $this->{version} ||= 3;

    #
    # Protocol options
    #
    # $this->{options} ||= search;

    $this->{id}      ||= undef;
    $this->{name}    ||= undef;
    $this->{version} ||= undef;

    bless $this;
    if ($this -> {sessionid})
      { return $this; }
    else
      { return 0; }
  }


#
# Try to initialize a Z39.50 association with a Z39.50 Target
#
sub Init
  {
    my $this      = shift;
    my $userid    = shift;
    my $passwd    = shift;
    my $reference = shift;

    my $status    = 0;
    my $reason    = 0;
    my $result    = 0;
    my $newreference = 0;


    if ($ZETA::VERBOSE)
      {
	print STDERR "\n>>>>>>>>>>>>>>>>>>>\n";
	print STDERR ">>> Init in ZETA.pm\n";
	print STDERR ">>> userid    = $userid\n";
	print STDERR ">>> password  = $passwd\n";
	print STDERR ">>> reference = $reference\n";
	print STDERR ">>>>>>>>>>>>>>>>>>>\n";
      }

    ($status, $reason, $result, $newreference) =
      ZETA::_init ($userid, $passwd, $reference, $ZETA::TIMEOUT);

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n<<<<<<<<<<<<<<<<<<<<<\n";
	print STDERR "<<< Init in ZETA.pm\n";
	print STDERR "<<< status    = $status\n";
	print STDERR "<<< reason    = $reason\n";
	print STDERR "<<< result    = $result\n";
	print STDERR "<<< reference = $newreference\n" if ($newreference);
	print STDERR "<<<<<<<<<<<<<<<<<<<<<\n";
      }

    $status, $reason, $result, $newreference
  }


#
# Try to search a Z39.50 Target for a query
#
sub Search
  {
    my $this      = shift;
    my $database  = shift;
    my $resultset = shift;
    my $query     = shift;
    my $setid     = shift;

    my $status    = 0;
    my $reason    = 0;
    my $howmany   = 0;
    my $info      = 0;

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n>>>>>>>>>>>>>>>>>>>>>\n";
	print STDERR ">>> Search in ZETA.pm\n";
	print STDERR ">>> database  = $database\n";
	print STDERR ">>> resultset = $resultset\n";
	print STDERR ">>> query     = \"$query\"\n";
	print STDERR ">>> attribute = \"$setid\"\n";
	print STDERR ">>>>>>>>>>>>>>>>>>>>>\n";
      }

    ($status, $reason, $howmany, $info) =
      ZETA::_search ($database, $resultset, $query, $setid, $ZETA::TIMEOUT);

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n<<<<<<<<<<<<<<<<<<<<<\n";
	print STDERR "<<< Search in ZETA.pm\n";
	print STDERR "<<< status  = $status\n";
	print STDERR "<<< reason  = $reason\n";
	print STDERR "<<< howmany = $howmany\n";
	print STDERR "<<< additional_search_info = $info\n" if ($info);
	print STDERR "<<<<<<<<<<<<<<<<<<<<<\n";
      }

    $status, $reason, $howmany, $info
  }


#
# Try to retrieve some records from a result set
#
sub Present
  {
    my $this      = shift;
    my $resultset = shift;
    my $items     = shift;
    my $start     = shift;
    my $esn       = shift;
    my $recstx    = shift;

    my $status    = 0;
    my $reason    = 0;
    my $howmany   = 0;
    my @records   = ();

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n>>>>>>>>>>>>>>>>>>>\n";
	print STDERR ">>> Present in ZETA.pm\n";
	print STDERR ">>> resultset = $resultset\n";
	print STDERR ">>> items     = $items\n";
	print STDERR ">>> start     = $start\n";
	print STDERR ">>> esn       = $esn\n";
	print STDERR ">>> recstx    = $recstx\n";
	print STDERR ">>>>>>>>>>>>>>>>>>>\n";
      }

    ($status, $reason, $howmany, @records) =
      ZETA::_present ($resultset, $items, $start, $esn, $recstx, $ZETA::TIMEOUT);

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n<<<<<<<<<<<<<<<<<<<<<<\n";
	print STDERR "<<< Present in ZETA.pm\n";
	print STDERR "<<< status  = $status\n";
	print STDERR "<<< reason  = $reason\n";
	print STDERR "<<< howmany = $howmany\n";
	print STDERR "<<< records = ", @records, "\n";
	print STDERR "<<<<<<<<<<<<<<<<<<<<<<\n";
      }

    $status, $reason, $howmany, @records
  }


#
# Try to delete a result set
#
sub Delete
  {
    my $this      = shift;
    my $function  = shift;
    my $resultset = shift;

    my $status    = 0;
    my $reason    = 0;

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n>>>>>>>>>>>>>>>>>>>>>\n";
	print STDERR ">>> Delete in ZETA.pm\n";
	print STDERR ">>> function = $function\n";
	print STDERR ">>> resultset = $resultset\n";
	print STDERR ">>>>>>>>>>>>>>>>>>>>>\n";
      }

    ($status, $reason) = ZETA::_delete ($function, $resultset, $ZETA::TIMEOUT);

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n<<<<<<<<<<<<<<<<<<<<<<\n";
	print STDERR "<<< Delete in ZETA.pm\n";
	print STDERR "<<< status = $status\n";
	print STDERR "<<< reason = $reason\n";
	print STDERR "<<<<<<<<<<<<<<<<<<<<<<\n";
      }

    $status, $reason
  }


#
# Try to scan a Z39.50 Target for an ordered term-list
#
sub Scan
  {
    my $this     = shift;
    my $database = shift;
    my $term     = shift;
    my $items    = shift;
    my $position = shift;

    my $status   = 0;
    my $reason   = 0;
    my $found    = 0;
    my @entries  = ();

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n>>>>>>>>>>>>>>>>>>>\n";
	print STDERR ">>> Scan in ZETA.pm\n";
	print STDERR ">>> database = $database\n";
	print STDERR ">>> term     = \"$term\"\n";
	print STDERR ">>> items    = $items\n";
	print STDERR ">>> position = $position\n";
	print STDERR ">>>>>>>>>>>>>>>>>>>\n";
      }

    ($status, $reason, $found, @entries) =
      ZETA::_scan ($database, $term, $items, $position, $ZETA::TIMEOUT);

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n<<<<<<<<<<<<<<<<<<<<<<\n";
	print STDERR "<<< Scan in ZETA.pm\n";
	print STDERR "<<< status  = $status\n";
	print STDERR "<<< reason  = $reason\n";
	print STDERR "<<< found   = $found\n";
	print STDERR "<<< entries = ", @entries, "\n";
	print STDERR "<<<<<<<<<<<<<<<<<<<<<<\n";
      }

    $status, $reason, $found, @entries
  }


#
# Try to sort a result set
#
sub Sort
  {
    my $this      = shift;
    my $resultset = shift;
    my $criteria  = shift;

    my $status    = 0;
    my $reason    = 0;

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n>>>>>>>>>>>>>>>>>>>>>\n";
	print STDERR ">>> Sort in ZETA.pm\n";
	print STDERR ">>> resultset = $resultset\n";
	print STDERR ">>> criteria  = $criteria\n";
	print STDERR ">>>>>>>>>>>>>>>>>>>>>\n";
      }

    ($status, $reason) = ZETA::_sort ($resultset, $criteria, $ZETA::TIMEOUT);

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n<<<<<<<<<<<<<<<<<<<<<\n";
	print STDERR "<<< Sort in ZETA.pm\n";
	print STDERR "<<< status = $status\n";
	print STDERR "<<< reason = $reason\n";
	print STDERR "<<<<<<<<<<<<<<<<<<<<<\n";
      }

    $status, $reason
  }


#
# Try to close a Z39.50 association
#
sub Close
  {
    my $this       = shift;
    my $reasoncode = shift;

    my $status    = 0;
    my $reason    = 0;

    if ($ZETA::VERBOSE)
      {
	print STDERR "\n>>>>>>>>>>>>>>>>>>>>>\n";
	print STDERR ">>> Close in ZETA.pm\n";
	print STDERR ">>> reasoncode = $reasoncode\n";
	print STDERR ">>>>>>>>>>>>>>>>>>>>>\n";
      }

    ($status, $reason) = ZETA::_close ($reasoncode, $ZETA::TIMEOUT);

    if ($ZETA::VERBOSE)
    {
      print STDERR "\n<<<<<<<<<<<<<<<<<<<<<\n";
      print STDERR "<<< Close in ZETA.pm\n";
      print STDERR "<<< status = $status\n";
      print STDERR "<<< reason = $reason\n";
      print STDERR "<<<<<<<<<<<<<<<<<<<<<\n";
    }

    $status, $reason
  }


package ZETA;

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

ZETA Perl - Simple ZETA 39.50 client interface

=head1 SYNOPSIS

  use ZETA;

  #constructor
  $zc = ZETA:Connection::New ($hostname, $port);

  #methods
  ($status, $reason, $result, $newreference) =
             $zc->Init ($user, $passwd, $id);

  ($status, $reason, $found, $info) =
             $zc->Search ($database, $resultset, $query, $setid);

  ($status, $reason, $returned, @records) =
             $zc->Present ($resultset, $howmany, $start, $format, $syntax);

  ($status, $reason) =
              $zc->Close ($closecode);

  ($status, $reason) =
              $zc->Delete ($function, $resultset);

  ($status, $reason, $returned, @entries) =
              $zc->Scan ($database, $term, $itemsno, $position);

  ($status, $reason) =
             $zc->Sort ($resultset, $criteria);


=head1 DESCRIPTION

This module implements a client interface to the Z39.50 protocol, enabling
a perl application to talk to Z39.50 servers.  It provides through an object
oriented interface, all basic operations offered by Z39.50 protocol to
its client.  This documentation assumes that you are familiar with the
concepts of the Z39.50 protocol.

The primary object is ZETA::Connection which is implemented as a blessed
hash reference.  The object supports `Init', `Search', `Present', `Close',
`Delete', `Scan', and `Sort' method calls.  For each method, the client sends
the appropriate request Z39.50 APDU to the server and waits for
a response. Each request is sent to the Z39.50 Target in synchronous
mode of operation; a method always waits for a response within a
configurable timeout whose default value is 360 seconds. This value can
be changed accordingly to user needs by setting the variable ZETA::TIMEOUT
to a different value (in seconds).


The latest version of the module is available at:

  http://zeta.tlcpi.finsiel.it/z3950/zetaperl/download.html

    or

  ftp://zeta.tlcpi.finsiel.it/pub/zeta/
  ftp://ftp.tecsiel.it/pub/zeta/


=head1 Notations and Conventions

       ZETA        static 'top-level' class name
       $zc         Connection object
       $status     Return variable (boolean) indicating the result of a method
       $reason     Return variable (string) containing the error message,
                   in case of failure, undef otherwise

       $attrid     The object identifier optionally used in a search method.
                   Attributes associated with a search term belong to
                   a registered attribute set identifier (an Object Identifier)
                   included with the query. Legal values for the Attribute Set
                   Id are the object identifiers registered in Appendix 3
                   of ANSI/NISO Z39.50-1995.

                   This module supports both the character string (e.g. bib1)
                   and the dot notation convention (e.g. 1.2.840.10003.3.1)

       $syntax     The object identifier optionally used in a present method.
                   Legal values for the presentation syntax are the object
                   identifiers registered in Appendix 5 of ANSI/NISO Z39.50-1995.

                   This module supports both the character string (e.g. grs1)
                   and the dot notation convention (e.g. 1.2.840.10003.5.105)

       $found      Number of items obtained in a search method
       $returned   Number of items returned in a scan or present method

       @records    List of values returned in a present method 
       @entries    List of values returned in a scan method

       $closecode  The code associated to a disconnection request.
                   It may have the following values and corrisponding meanings:

                     1 [ finished ]
                     2 [ shutdown ]
                     3 [ system problem]
                     4 [ cost limit ]
                     5 [ resources ]
                     6 [ security violation ]
                     7 [ protocol error ]
                     8 [ lack of activity ]
                     9 [ peer abort ]
                    10 [ unspecified ]



=head1 INSTALLATION

To install ZETA Perl extention module, just change to the directory in
which this file is found and type the following:

	perl Makefile.PL
	make
	make install

This will copy ZETA.pm to your perl library directory for use by all Perl
scripts. You probably must be root to do this. Once installed this module,
you can load the ZETA Perl routines in your Perl scripts with the line:

	use ZETA;

If you don't have sufficient privileges to install ZETA.pm in the Perl
library directory, you can put ZETA.pm into some convenient place, such
as your home directory and prefix all Perl scripts that call it with
something along the lines of the following preamble:

	use lib '/full_path_of_my_home_dir/';
	use ZETA;


Before or after you install this module, you can perform some tests to
check the whole functionalities of the module with the following command:

        make test

However, in order to test the integrity and functionalities of this module
you need to have a reachable z3950 target in somewhere on the net.  The
system wide configuration file, zetaperlrc, which comes with this package,
contains default values for testing.


=head1 CONSTRUCTOR

=over 5

=item B<New ZETA::Connection (Host, Port);>

This is the constructor for a new ZETA:Connection object.  <Host> is the
hostname or IP address in dot notation of the Z39.50 target to which a
ZETA connection is required, and <Port> is the port number.  Both <Host>
and <Port> are required parameters.

Return value

This function returns NULL value when an error occurs. The reason of failure
can be:

  (1) No more file descriptors are available in the system for this process;
  (2) The connection cannot be established at the specified address.

Example:

    $Host = "host.domain.somevalid";
    $Port = 210;  # NISO Z39.50 database
    $zc = ZETA::Connection::New ($Host, $Port);
    if ($zc)
     {
        print "Connection established";
     }
    else
     {
       print "Connection failed";
     }

=head1 ERROR HANDLING

If not otherwise stated, all methods return in the output variable $status
an indication if the function succeeds. Upon a successful completion, the
return variable <$status> is set to the value 1, and the output variable
$reason is left undefined.  Otherwise, each time the method fails the
output variable $status is set to 0 (zero) and the output variable
$reason contains a text message indicating the reason of the failure.


=head1 METHODS

=item B<Init (user, passwd, refid);>

     ($status, $reason, $result, $newid) =
                        $zc->Init (user, passwd, refid);

Sends a request to the server to initizialize a previously established
connection identified by <refid>.  The first two parameters serve to transmit
the authentication information (in case your target applies).  As Z39.50
servers may or may not require authentication, one may pass these two
paramateres as undef values.  If no <user> and <passwd> are given, the
NULL string is used for both parameters.

Return value

This method returns an array of four elements with the following meanings:

$status: see error handling statements.

$reason: see error handling statements.

$result: the indication if the Z39.50 association was accepted or rejected

$reference: new reference for the connection

=item B<Search (database, resultset, query, attrid);>

     ($status, $reason, $found, $info) =
          $zc->Search (database, resultset, query, attrid);

Sends a request to the server to make Z39.50 Type 1 queries which are
general purpose Boolean query structures.  The first parameter,
<database>, indentifies the name of database to which the query
is required. The <resultset> indicates the name to give to the
query result, while <query> is the query structure (see below.)
The last  parameter <attrid>, which indicates the attribute set identifier
of the client, is optional.

Queries must be constructed according to Z39.50 basic definitions.  In this
case the search term, <query>, must be in a Type-1 query form, that means
in the form of AttributeList+Term (as defined in section 3.7.1 of
Z39.50-1995).  For example, if you want to search a word, say "Dante",
in the title field of your database, you must construct your query
as follows:

                   $query = "1=4 Dante";

In general,
                   $query = "AttributeType=AttributeValue  Word"

So in the previous example, AttributeType=1 (that means USE Attribute), and
the AttributeValue=4 (that means Title).  One may want to construct more
complicated query strings by using different Attribute types and values,
and their combinations with Boolean operators.  The latest version of
Z39.50 (V.3 1995), allows powerful search statements.  For more information on
how to construct complex queries, refer to section 3.7.1 of Z39.50-1995)

Return value

This method returns an array of four elements with the following meanings:

$status: see error handling statements.

$reason: see error handling statements.

$found: the total number of items in the resultset, if one has been created.

$info: Additional information. This return value depends on the Z39.50 target.



=item B<Present (resultset, howmany, start, format, syntax)>

   ($status, $reason, $returned, @records) =
          $zc->Present (resultset, howmany, start, format, syntax);

Retrieves some records from a previously created resultset. The first
parameter identifies the resultset for which data retrieval is required.
The second and third parameters specify respectively the maximum number
of items to be retrieved, and the record number from which to begin data
retreiving. The fourth parameter, <format>, indicates the Element Set
Name (`b' for brief, `mb' for museum brief, `f' for full, etc.), while
the last parameter, <syntax>, indicates the record syntax of records
being retreived.


Return value

This method returns an array of four elements with the following meanings:

$status: see error handling statements.

$reason: see error handling statements.

$returned: the total number of items being retrieved, if any.

@records: array containing the elements rerteived, if any.


=item  B<Delete (function, resultset)>

    ($status, $reason) =
          $zc->Delete ($function, $resultset);

Sends a request to the server for deleting a particular resultset, or
a list of resultesets.  The <function> parameter may assume one of the
following  values:
            Z_DELETE_LIST = 1
            Z_DELETE_ALL  = 2

Return value

This method returns two elements as follows:

$status: see error handling statements.

$reason: see error handling statements.


=item B<Close (closecode)>

         ($status, $reason) = Close (closecode);

Sends a request to close a previously established connection.  It does not
check for any state information, therefore it may be called at any time
to communicate that the user is finished with the Z39.50 association.  The only
parameter, which is required, is the reason code of the disconnection.

This method returns two elements as follows:

$status: see error handling statements.

$reason: see error handling statements.


=item B<Scan (database, term, itemsno, position)>

    ($status, $reason, $returned, @entries) =
          $zc->Scan (database, term, itemsno, position);

Sends a request to the server for scan an ordered term-list.  The <database>
name and the <term> for scanning parameters are required.  The maximum number
of items to retrieve <itemsno> and the item number from which to begin data
retreival <position> are optional.

Return Value
This method returns four elements as follows:

$status: see error handling statements.

$reason: see error handling statements.

$returned: the total number of items retreived, if any.

@entries: an array containing the items retrieved, if any.  Each element of
          this array contains th standard retrieved entry information in the
          following format: "term (num)", where <term> is the element found,
          and <num> is the occurance number related to this element.



=item B<Sort ($resultset, $criteria)>

      ($status, $reason) = Sort (resultset, criteria)

Sends a request to the server for sorting a previously created resultset
<resultset>.  The <criteria> indicates the sort sequences criteria. Both
the two parameters are required.

Return Value

This method returns two elements as follows:

$status: see error handling statements.

$reason: see error handling statements.



=head1 BUGS

No bugs are known at this time.

=head1 AUTHORS

ZETA Perl has been primarily written by Rocco Carbone <rocco@tecsiel.it>,
who has been helped by Jama Musse Jama <jama@tecsiel.it>,
Massimo Zalloco <mz@tecsiel.it> and Maria Grazia Toro <grazia@tecsiel.it>.
Jama ported this module in win32 platform and contribued the great part of
the ducamentation.

=head1 COPYRIGHT

Copyright (c): 1996, 1999 Finsiel S.p.A.
This program is free software; you can redistribute it and/or modify under
the same terms as Perl itself.


=cut
